@@ -3,7 +3,7 @@
 
 double baz (double x, double y, double z, double u)
 {
-  return x * x * x * y * y * y * z * z * z * z * u * u * u * u;
+  return x * x * y * y * y * z * z * z * z * u;
 }
 
 /* { dg-final { scan-tree-dump-times " \\* " 6 "optimized" } } */
