@@ -11,6 +11,7 @@ struct test {
 
 extern struct test s;
  
+__attribute__ ((noinline))
 int main1 ()
 {  
   int i;
@@ -32,8 +33,9 @@ int main1 ()
 
 int main (void)
 { 
+  check_vect ();
+
   return main1 ();
 } 
 
-/* { dg-final { scan-tree-dump-times "vectorization not profitable" 1 "vect" } } */
-/* { dg-final { cleanup-tree-dump "vect" } } */
+/* { dg-final { scan-tree-dump-times "vectorized 1 loops" 1 "vect" } } */
