@@ -19,9 +19,10 @@ int main1 ()
   for (i=0; i < N+1; i++)
     {
       ib[i] = i;
-      /* Avoid vectorization.  */
       if (i%3 == 0)
         ib[i] = 5;
+      /* Avoid vectorization.  */
+      __asm__ volatile ("" : : : "memory");
     }
 
   for (i = 1; i < N+1; i++)
@@ -49,4 +50,3 @@ int main (void)
 /* { dg-final { scan-tree-dump-times "vectorized 1 loops" 1 "vect" { xfail { vect_no_align && { ! vect_hw_misalign } } } } } */
 /* { dg-final { scan-tree-dump-times "Vectorizing an unaligned access" 1 "vect" { xfail { vect_no_align && { ! vect_hw_misalign } } } } } */
 /* { dg-final { scan-tree-dump-times "Alignment of access forced using peeling" 0 "vect" } } */
-/* { dg-final { cleanup-tree-dump "vect" } } */
