@@ -1,12 +1,12 @@
 /* { dg-do compile } */
-/* { dg-skip-if "" { ! { arm_thumb1_ok || arm_thumb2_ok } } } */
+/* { dg-require-effective-target arm_thumb2_ok } */
 /* { dg-options "-mthumb -Os" }  */
-/* { dg-final { scan-assembler "push\t\{r3" } } */
-/* { dg-final { scan-assembler-not "r8" } } */
+/* { dg-final { scan-assembler "push\t.*r8" } } */
+/* { dg-final { scan-assembler-not "push\t*r3" } } */
 
 extern int hist_verify;
 extern char *pre_process_line (char*);
-extern char* savestring1 (char*, char*);
+extern char* savestring1 (char*, char*, int, int);
 extern char* str_cpy (char*, char*);
 extern int str_len (char*);
 extern char* x_malloc (int);
@@ -22,6 +22,7 @@ history_expand_line_internal (char* line)
   hist_verify = 0;
   new_line = pre_process_line (line);
   hist_verify = old_verify;
-  /* Two tail calls here, but r3 is not used to pass values.  */
-  return (new_line == line) ? savestring (line) : savestring1 (new_line, line);
+  /* Two tail calls here, but r3 is used to pass values.  */
+  return (new_line == line) ? savestring (line) : 
+    savestring1 (new_line, line, 0, old_verify+1);
 }
