@@ -5,8 +5,8 @@ int main ()
 {
   int a;
 
-  a = __builtin_bfin_norm_fr1x32 (0);
-  if (a != 31)
+  a = __builtin_bfin_norm_fr1x32 (0x12345678);
+  if (a != 2)
     abort ();
 
   exit (0);
