@@ -19,14 +19,14 @@ main ()
     return 0;
 
   /* Run AVX2 test only if host has AVX2 support.  */
-  if ((ecx & bit_OSXSAVE) == (bit_OSXSAVE))
+  if (ecx & bit_OSXSAVE)
     {
       if (__get_cpuid_max (0, NULL) < 7)
 	return 0;
 
       __cpuid_count (7, 0, eax, ebx, ecx, edx);
 
-      if ((avx_os_support ()) && ((ebx & bit_AVX2) == bit_AVX2))
+      if ((ebx & bit_AVX2) && avx_os_support ())
 	{
 	  do_test ();
 #ifdef DEBUG
