@@ -34,12 +34,12 @@ void fn_default_start (void) { }
  **
  **/
 
-__attribute__ ((target ("stack-guard=0")))
+__attribute__ ((target ("arch=z10")))
 void fn_att_0 (void) { }
-/* { dg-final { scan-assembler "fn:fn_att_0 sg0" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0 ar6" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_att_0 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0 hd0" } } */
@@ -55,12 +55,12 @@ void fn_att_0 (void) { }
 
 void fn_att_0_default (void) { }
 
-__attribute__ ((target ("stack-guard=16")))
+__attribute__ ((target ("arch=z13")))
 void fn_att_1 (void) { }
-/* { dg-final { scan-assembler "fn:fn_att_1 sg16" } } */
-/* { dg-final { scan-assembler "fn:fn_att_1 ar6" } } */
+/* { dg-final { scan-assembler "fn:fn_att_1 ar9" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_att_1 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1 hd0" } } */
@@ -76,12 +76,12 @@ void fn_att_1 (void) { }
 
 void fn_att_1_default (void) { }
 
-__attribute__ ((target ("stack-guard=16,stack-guard=0")))
+__attribute__ ((target ("arch=z13,arch=z10")))
 void fn_att_1_0 (void) { }
-/* { dg-final { scan-assembler "fn:fn_att_1_0 sg0" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1_0 ar6" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1_0 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1_0 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_att_1_0 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1_0 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1_0 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1_0 hd0" } } */
@@ -95,12 +95,12 @@ void fn_att_1_0 (void) { }
 /* { dg-final { scan-assembler "fn:fn_att_1_0 mv1" } } */
 /* { dg-final { scan-assembler "fn:fn_att_1_0 wd0" } } */
 
-__attribute__ ((target ("stack-guard=0,stack-guard=16")))
+__attribute__ ((target ("arch=z10,arch=z13")))
 void fn_att_0_1 (void) { }
-/* { dg-final { scan-assembler "fn:fn_att_0_1 sg16" } } */
-/* { dg-final { scan-assembler "fn:fn_att_0_1 ar6" } } */
+/* { dg-final { scan-assembler "fn:fn_att_0_1 ar9" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0_1 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0_1 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_att_0_1 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0_1 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0_1 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_att_0_1 hd0" } } */
@@ -120,12 +120,12 @@ void fn_att_0_1 (void) { }
  **
  **/
 
-#pragma GCC target ("stack-guard=0")
+#pragma GCC target ("arch=z10")
 void fn_pragma_0 (void) { }
-/* { dg-final { scan-assembler "fn:fn_pragma_0 sg0" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0 ar6" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_0 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0 hd0" } } */
@@ -158,12 +158,12 @@ void fn_pragma_0_default (void) { }
 /* { dg-final { scan-assembler "fn:fn_pragma_0_default mv1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_default wd0" } } */
 
-#pragma GCC target ("stack-guard=16")
+#pragma GCC target ("arch=z13")
 void fn_pragma_1 (void) { }
-/* { dg-final { scan-assembler "fn:fn_pragma_1 sg16" } } */
-/* { dg-final { scan-assembler "fn:fn_pragma_1 ar6" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_1 ar9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_1 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1 hd0" } } */
@@ -196,13 +196,13 @@ void fn_pragma_1_default (void) { }
 /* { dg-final { scan-assembler "fn:fn_pragma_1_default mv1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_default wd0" } } */
 
-#pragma GCC target ("stack-guard=16")
-#pragma GCC target ("stack-guard=0")
+#pragma GCC target ("arch=z13")
+#pragma GCC target ("arch=z10")
 void fn_pragma_1_0 (void) { }
-/* { dg-final { scan-assembler "fn:fn_pragma_1_0 sg0" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_0 ar6" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_0 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_0 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_1_0 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_0 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_0 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_0 hd0" } } */
@@ -217,13 +217,13 @@ void fn_pragma_1_0 (void) { }
 /* { dg-final { scan-assembler "fn:fn_pragma_1_0 wd0" } } */
 #pragma GCC reset_options
 
-#pragma GCC target ("stack-guard=0")
-#pragma GCC target ("stack-guard=16")
+#pragma GCC target ("arch=z10")
+#pragma GCC target ("arch=z13")
 void fn_pragma_0_1 (void) { }
-/* { dg-final { scan-assembler "fn:fn_pragma_0_1 sg16" } } */
-/* { dg-final { scan-assembler "fn:fn_pragma_0_1 ar6" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_0_1 ar9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_1 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_1 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_0_1 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_1 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_1 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_1 hd0" } } */
@@ -244,13 +244,13 @@ void fn_pragma_0_1 (void) { }
  **
  **/
 
-#pragma GCC target ("stack-guard=0")
-__attribute__ ((target ("stack-guard=0")))
+#pragma GCC target ("arch=z10")
+__attribute__ ((target ("arch=z10")))
 void fn_pragma_0_att_0 (void) { }
-/* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 sg0" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 ar6" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 hd0" } } */
@@ -265,13 +265,13 @@ void fn_pragma_0_att_0 (void) { }
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_0 wd0" } } */
 #pragma GCC reset_options
 
-#pragma GCC target ("stack-guard=0")
-__attribute__ ((target ("stack-guard=0")))
+#pragma GCC target ("arch=z10")
+__attribute__ ((target ("arch=z10")))
 void fn_pragma_1_att_0 (void) { }
-/* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 sg0" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 ar6" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 hd0" } } */
@@ -286,13 +286,13 @@ void fn_pragma_1_att_0 (void) { }
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_0 wd0" } } */
 #pragma GCC reset_options
 
-#pragma GCC target ("stack-guard=0")
-__attribute__ ((target ("stack-guard=16")))
+#pragma GCC target ("arch=z10")
+__attribute__ ((target ("arch=z13")))
 void fn_pragma_0_att_1 (void) { }
-/* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 sg16" } } */
-/* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 ar6" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 ar9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 hd0" } } */
@@ -307,13 +307,13 @@ void fn_pragma_0_att_1 (void) { }
 /* { dg-final { scan-assembler "fn:fn_pragma_0_att_1 wd0" } } */
 #pragma GCC reset_options
 
-#pragma GCC target ("stack-guard=0")
-__attribute__ ((target ("stack-guard=16")))
+#pragma GCC target ("arch=z10")
+__attribute__ ((target ("arch=z13")))
 void fn_pragma_1_att_1 (void) { }
-/* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 sg16" } } */
-/* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 ar6" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 ar9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 tu9" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 ss2048" } } */
+/* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 sg16" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 bc1" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 wf512" } } */
 /* { dg-final { scan-assembler "fn:fn_pragma_1_att_1 hd0" } } */
