@@ -6,7 +6,7 @@ program streamtest
   character(1)   :: tchar
   integer        :: i,j,k
   real(kind=4), dimension(100,100) :: anarray
-  open(10, file="teststream", access="stream", form="unformatted")
+  open(10, file="teststream_streamio_12", access="stream", form="unformatted")
   anarray = 3.14159
   write(10) anarray
   write(10, pos=1) ! This is a way to position an unformatted file
