@@ -15,8 +15,7 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-// { dg-options "-std=gnu++11" }
-// { dg-do compile }
+// { dg-do compile { target c++11 } }
 
 #include <memory>
 
@@ -30,3 +29,12 @@ const std::owner_less<std::weak_ptr<int>> owi;
 static_assert( noexcept(owi(wi, wi)), "" );
 static_assert( noexcept(owi(si, wi)), "" );
 static_assert( noexcept(owi(wi, si)), "" );
+const std::shared_ptr<long> sl;
+const std::weak_ptr<char> wc;
+const std::owner_less<void> ov;
+static_assert( noexcept(ov(si, si)), "" );
+static_assert( noexcept(ov(si, sl)), "" );
+static_assert( noexcept(ov(sl, si)), "" );
+static_assert( noexcept(ov(si, wc)), "" );
+static_assert( noexcept(ov(wc, si)), "" );
+static_assert( noexcept(ov(wc, wi)), "" );
