@@ -1,6 +1,6 @@
 // 2005-08-29  Paolo Carlini  <pcarlini@suse.de>
 //
-// Copyright (C) 2005-2015 Free Software Foundation, Inc.
+// Copyright (C) 2005-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -22,8 +22,7 @@
 
 // libstdc++/23578
 void test01() 
-{ 
-  bool test __attribute__((unused)) = true;
+{
   typedef std::vector<int> vector_type;
 
   {
