@@ -1,4 +1,4 @@
-// Copyright (C) 2003-2016 Free Software Foundation, Inc.
+// Copyright (C) 2003-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -54,19 +54,16 @@ void test01()
 {
   using namespace std;
 
-  bool test __attribute__((unused)) = true;
   const char* name = "tmp_close_12790";
 
   Cvt* cvt = new Cvt;
   locale loc(locale::classic(), cvt);
 
   wfilebuf fb;
-  fb.pubsetbuf(0, 0);
   fb.pubimbue(loc);
 
   fb.open(name, ios_base::out);
   fb.sputc(L'a');
-  fb.in_avail(); // showmanyc() should have no effect on close().
 
   VERIFY( !cvt->unshift_called );
   fb.close();
