@@ -22,28 +22,30 @@
 #include <fstream>
 #include <testsuite_hooks.h>
 
-const char name_01[] = "tmp_11389-1";
+const char name_02[] = "tmp_11389-2";
 
-void test01()
+void test02()
 {
   using namespace std;
 
   filebuf fbout;
-  fbout.open(name_01, ios_base::out);
+  fbout.open(name_02, ios_base::out);
   fbout.sputc('a');
+  fbout.sputc('b');
   fbout.close();
   
   wfilebuf fbin;
   locale loc(locale("en_US.UTF-8"));
   fbin.pubimbue(loc);
-  fbin.open(name_01, ios_base::in);
+  fbin.open(name_02, ios_base::in);
   VERIFY( fbin.sbumpc() == L'a' );
+  VERIFY( fbin.sbumpc() == L'b' );
   VERIFY( fbin.sgetc() == wfilebuf::traits_type::eof() );
   fbin.close();
 }
 
 int main()
 {
-  test01();
+  test02();
   return 0;
 }
