@@ -1,6 +1,6 @@
-// 2004-01-11  Petur Runolfsson  <peturr02@ru.is>
+// 2003-05-01  Petur Runolfsson  <peturr02@ru.is>
 
-// Copyright (C) 2004-2017 Free Software Foundation, Inc.
+// Copyright (C) 2000-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -18,42 +18,16 @@
 // <http://www.gnu.org/licenses/>.
 
 #include <iostream>
-#include <string>
-#include <locale>
 
-// libstdc++/13582
-void test01()
+void test09()
 {
-  using namespace std;
-
-  ios_base::sync_with_stdio(false);
-  wcout << "Type in 12345\n";
-  
-  wstring str;
-  wchar_t c;
-  
-  if (wcin.get(c) && !isspace(c, wcin.getloc()))
-    {
-      str.push_back(c);
-      wcin.imbue(locale(ISO_8859(1,en_US)));
-    }
-
-  if (wcin.get(c) && !isspace(c, wcin.getloc()))
-    {
-      str.push_back(c);
-      wcin.imbue(locale(ISO_8859(15,fr_FR)));
-    }
-
-  while (wcin.get(c) && !isspace(c, wcin.getloc()))
-    {
-      str.push_back(c);
-    }
-  
-  wcout << str << endl;
+  std::wcout << L"Enter favorite beach: ";
+  std::wcin.ignore(2048, '\n');
 }
 
-int main()
+int 
+main()
 {
-  test01();
+  test09();
   return 0;
 }
