@@ -1,8 +1,8 @@
-// { dg-options "-std=gnu++11" }
+// { dg-do run { target c++11 } }
 //
 // 2009-06-17  Stephen M. Webb  <stephen.webb@xandros.com>
 //
-// Copyright (C) 2009-2015 Free Software Foundation, Inc.
+// Copyright (C) 2009-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -27,18 +27,16 @@
 void
 test01()
 {
-  bool test __attribute__((unused)) = true;
+  std::regex_error error(std::regex_constants::error_collate);
+  VERIFY(error.code() == std::regex_constants::error_collate);
 
-	std::regex_error error(std::regex_constants::error_collate);
-	VERIFY(error.code() == std::regex_constants::error_collate);
-
-	try
-	{
-		throw error;
-	}
-	catch (std::runtime_error& ex)
-	{
-	}
+  try
+    {
+      throw error;
+    }
+  catch (std::runtime_error& ex)
+    {
+    }
 }
 
 int main()
