@@ -1,9 +1,8 @@
-// { dg-do run }
-// { dg-options "-std=gnu++0x" }
+// { dg-do run { target c++11 } }
 
 // 2007-03-12  Stephen M. Webb  <stephen.webb@bregmasoft.com>
 //
-// Copyright (C) 2010-2013 Free Software Foundation, Inc.
+// Copyright (C) 2010-2017 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -28,7 +27,6 @@
 // Verifies the workings of the regex_traits translate_nocase function.
 void test01()
 {
-  bool test __attribute__((unused)) = true;
   typedef char CharT;
 
   std::regex_traits<CharT> t;
@@ -43,4 +41,4 @@ main()
 { 
   test01();
   return 0;
-};
+}
