@@ -21,13 +21,12 @@
 
 #include <testsuite_hooks.h>
 
-// libstdc++/7986
+// c++/3111
 int main()
 {
   using namespace __gnu_test;
 
-  // cplus-dem FAIL  
-  verify_demangle("_Z1jM1AFivEPS1_", "j(int (A::*)(), int (A::**)())");
+  verify_demangle("_Z1fKPFiiE", "f(int (* const)(int))");
 
   return 0;
 }
